@@ -8,7 +8,7 @@ class UnauthorizedAccessException(Exception):
     def __init__(self, message):
         self.message = message
 
-class CDriveClient:
+class Client:
     def __init__(self, domain=None, username=None, password=None, access_token=None):
         if domain != None:
             self.api_url = 'https://api.' + domain + '/'
@@ -20,51 +20,61 @@ class CDriveClient:
         elif (username != None and password != None):
             self.access_token = self.get_token(username, password)
             self.home = 'users/' + username
+        elif 'COLUMBUS_ACCESS_TOKEN' in os.environ:
+            self.access_token = os.environ['COLUMBUS_ACCESS_TOKEN']
+            self.home = 'users/' + self.user_details()['username']
         else:
             self.access_token = self.get_token(os.environ['COLUMBUS_USERNAME'], os.environ['COLUMBUS_PASSWORD'])
+
     def get_token(self, username, password):
         response = requests.post(url= self.api_url + 'api-access-token/', data={'username': username, 'password': password})
         return response.json()['accessToken']
+
     def user_details(self):
         auth_header = 'Bearer ' + self.access_token
         response = requests.get(url= self.api_url + 'user-details/', headers={'Authorization': auth_header})
         if response.status_code == 401:
             raise UnauthorizedAccessException('Cannot identify user')
         return response.json()
-    def upload(self, cdrive_path, **kwargs):
-        if not ('content' in kwargs or 'local_path' in kwargs):
-            raise Exception('Either content or local_path argument needs to be provided')
-        elif ('content' in kwargs and 'local_path' in kwargs):
-            raise Exception('Only one of content or local_path arguments should be provided')
-        elif ('local_path' in kwargs and os.path.isdir(kwargs['local_path'])):
-            local_path = kwargs['local_path']
-            data = {
-                'path': cdrive_path,
-                'name': os.path.basename(local_path)
-            }
-            requests.post(self.api_url + 'create/', data=data, headers={'Authorization': 'Bearer ' + self.access_token})
+
+    def upload(self, local_path, cdrive_path):
+        if os.path.isdir(local_path):
+            folder_name = os.path.basename(local_path)
+            self.create_folder(cdrive_path, folder_name)
             for obj_name in os.listdir(local_path):
-                self.upload(cdrive_path + '/' + data['name'], local_path=os.path.join(local_path, obj_name))
+                self.upload(cdrive_path + '/' + folder_name, os.path.join(local_path, obj_name))
         else:
             file_arg = None
-            if 'local_path' in kwargs:
-                f = open(kwargs['local_path'], 'rb')
-                file_name = os.path.basename(local_path)
-                file_arg = {'file': (file_name, f), 'path': (None, cdrive_path)}
-                requests.post(self.api_url + 'upload/', files=file_arg, headers={'Authorization': 'Bearer ' + self.access_token})
-                f.close()
-            elif 'file_name' not in kwargs:
-                raise Exception('file_name argument must be provided along with content argument')
-            else:
-                file_arg = {'file': (kwargs['file_name'], kwargs['content']), 'path': (None, cdrive_path)}
-                requests.post(self.api_url + 'upload/', files=file_arg, headers={'Authorization': 'Bearer ' + self.access_token})
-    def create(self, cdrive_path, name):
+            f = open(local_path, 'rb')
+            file_name = os.path.basename(local_path)
+            file_arg = {'file': (file_name, f), 'path': (None, cdrive_path)}
+            response = requests.post(self.api_url + 'upload/', files=file_arg, headers={'Authorization': 'Bearer ' + self.access_token})
+            f.close()
+            if response.status_code == 401:
+                raise UnauthorizedAccessException(response.json()['message'])
+            elif response.status_code == 403:
+                raise ForbiddenAccessException(response.json()['message'])
+
+    def create_file(self, cdrive_path, file_name, content):
+        file_arg = {'file': (file_name, content), 'path': (None, cdrive_path)}
+        response = requests.post(self.api_url + 'upload/', files=file_arg, headers={'Authorization': 'Bearer ' + self.access_token})
+        if response.status_code == 401:
+            raise UnauthorizedAccessException(response.json()['message'])
+        elif response.status_code == 403:
+            raise ForbiddenAccessException(response.json()['message'])
+
+    def create_folder(self, cdrive_path, name):
         data = {
             'path': cdrive_path,
             'name': name
         }
         response = requests.post(self.api_url + 'create/', data=data, headers={'Authorization': 'Bearer ' + self.access_token})
-    def list(self, cdrive_path, recursive=False):
+        if response.status_code == 401:
+            raise UnauthorizedAccessException(response.json()['message'])
+        elif response.status_code == 403:
+            raise ForbiddenAccessException(response.json()['message'])
+
+    def list_detailed(self, cdrive_path):
         response = requests.get(self.api_url + 'list/?path=' + cdrive_path, headers={'Authorization': 'Bearer ' + self.access_token})
         if response.status_code == 401:
             raise UnauthorizedAccessException('Cannot identify user')
@@ -73,42 +83,72 @@ class CDriveClient:
         elif response.status_code != 200:
             raise Exception('Error reading from CDrive')
         return response.json()
-    def download(self, cdrive_path, **kwargs):
-        folder = False
-        if 'folder' not in kwargs:
-            parent_path = cdrive_path[:cdrive_path.rfind('/')]
-            dobjs = self.list(parent_path)
-            obj_it = filter(lambda x: x['name'] == cdrive_path[cdrive_path.rfind('/') + 1:], dobjs['driveObjects'])
-            try:
-                dobj = next(obj_it)
+
+    def list(self, cdrive_path)
+        return map(lambda dobj: dobj['name'], self.list_detailed(cdrive_path))
+
+    def list_files(self, cdrive_path, recursive=True):
+        if recursive:
+            return self.list_files_recursive(cdrive_path, '')
+        else:
+            files = []
+            folder_details = self.list_detailed(cdrive_path)
+            for dobj in folder_details['driveObjects']:
                 if dobj['type'] == 'File':
-                    folder = False
-                elif dobj['type'] == 'Folder':
-                    folder = True
-            except StopIteration:
-                raise ForbiddenAccessException('Requested object does not exist or you do not have permission to access it')
-        else:
-            folder = kwargs['folder']
-        if folder:
-            child_objs = self.list(cdrive_path)['driveObjects']
-            for cobj in child_objs:
-                local_path = os.path.join(kwargs['local_path'], cdrive_path[cdrive_path.rfind('/') + 1:])
-                self.download(cdrive_path + '/' + cobj['name'], local_path=local_path, folder=(cobj['type']=='Folder'))
-        else:
-            response = requests.get(self.api_url + 'download/?path=' + cdrive_path, headers={'Authorization':'Bearer ' + self.access_token})
-            if response.status_code != 200:
-                raise Exception('Error in Download')
-            download_url = response.json()['download_url']
-            if 'local_path' not in kwargs:
-                return download_url
+                    files.append(dobj['name'])
+            return files
+
+    def list_files_recursive(self, cdrive_path, base_path):
+        folder_details = self.list_detailed(cdrive_path)
+        files = []
+        for dobj in folder_details['driveObjects']:
+            if dobj['type'] == 'Folder':
+                files.extend(self.list_files_recursive(cdrive_path + '/' + dobj['name'], base_path + '/' + dobj['name']))
             else:
-                url_wo_sig = download_url[:download_url.find('?')]
-                name = url_wo_sig[url_wo_sig.rfind('/') + 1 :]
-                os.makedirs(kwargs['local_path'], exist_ok=True)
-                file_path = os.path.join(kwargs['local_path'], name)
-                response = requests.get(download_url)
-                with open(file_path, 'wb') as f:
-                    f.write(response.content)
+                files.append(base_path + '/' + dobj['name'])
+        return files
+
+    def download_folder(self, cdrive_path, local_path):
+        child_objs = self.list_detailed(cdrive_path)['driveObjects']
+        folder_path = os.path.join(local_path, cdrive_path[cdrive_path.rfind('/')+1:])
+        os.mkdir(folder_path)
+        for cobj in child_objs:
+            if cobj['type'] == 'Folder':
+                download_folder(cdrive_path + '/' + cobj['name'], folder_path)
+            else:
+                download_file(cdrive_path + '/' + cobj['name'], folder_path)
+
+    def download_file(self, cdrive_path, local_path):
+        file_url = self.file_url(cdrive_path)
+        name = cdrive_path[cdrive_path.rfind('/')+1:]
+        response = requests.get(file_url)
+        with open(os.path.join(local_path, name), 'wb') as f:
+            f.write(response.content)
+
+    def download(self, cdrive_path, local_path):
+        parent_path = cdrive_path[:cdrive_path.rfind('/')]
+        dobjs = self.list_detailed(parent_path)
+        obj_it = filter(lambda x: x['name'] == cdrive_path[cdrive_path.rfind('/') + 1:], dobjs['driveObjects'])
+        try:
+            dobj = next(obj_it)
+            if dobj['type'] == 'File':
+                download_file(cdrive_path, local_path)
+            else:
+                download_folder(cdrive_path, local_path)
+        except StopIteration:
+            raise ForbiddenAccessException('Requested object does not exist or you do not have permission to access it')
+
+    def file_url(self, cdrive_path):
+        response = requests.get(self.api_url + 'download/?path=' + cdrive_path, headers={'Authorization':'Bearer ' + self.access_token})
+        if response.status_code == 403:
+            raise ForbiddenAccessException('Requested object does not exist or you do not have permission to access it')
+        elif response.status_code == 401:
+            raise UnauthorizedAccessException('Cannot identify user')
+        elif response.status_code != 200:
+            raise Exception('Error in Download')
+        else:
+            return response.json()['download_url']
+
     def delete(self, cdrive_path):
         response = requests.delete(self.api_url + 'delete/?path=' + cdrive_path, headers={'Authorization':'Bearer ' + self.access_token})
         if response.status_code == 401:
@@ -117,6 +157,7 @@ class CDriveClient:
             raise ForbiddenAccessException('The object does not exist or you do not have permission to delete it')
         elif response.status_code != 204:
             raise Exception('Error reading from CDrive')
+
     def share(self, cdrive_path, permission, target_name='', target_type='user'):
         data = {
             'path': cdrive_path,
@@ -125,6 +166,7 @@ class CDriveClient:
             'name': target_name
         }
         response = requests.post(self.api_url + 'share/', data=data, headers={'Authorization': 'Bearer ' + self.access_token})
+
     def install_app(self, app_url):
         response = requests.post(self.api_url + 'install-application/', data={'app_docker_link': app_url}, headers={'Authorization': 'Bearer ' + self.access_token})
         if response.status_code == 201:
@@ -134,6 +176,7 @@ class CDriveClient:
                 if res.status_code == 200 and res.json()['appStatus'] == 'Available':
                     break
             return app_name
+
     def app_token(self, app_name):
         response = requests.post(self.api_url + 'app-token/', data={'app_name': app_name}, headers={'Authorization': 'Bearer ' + self.access_token})
         if response.status_code == 200:
